popover#puzzles-popover {
  padding: 10px;
}

stack#stack-packs > label {
  font-weight: bold;
}

.treeview-container treeview {
  background-color: transparent;
}
.treeview-container treeview:backdrop {
  color: @theme_unfocused_fg_color;
}
.treeview-container treeview:hover {
  background-color: @theme_bg_color;
}
.treeview-container treeview:hover:backdrop {
  background-color: @theme_unfocused_bg_color;
}
.treeview-container treeview:selected,
.treeview-container treeview:selected:hover {
  background-color: @theme_selected_bg_color;
  color: @theme_selected_fg_color;
}
.treeview-container treeview:selected:backdrop,
.treeview-container treeview:selected:backdrop:hover {
  background-color: @theme_unfocused_selected_bg_color;
  color: @theme_unfocused_selected_fg_color;
}

treeview.toggle-like-pixbuf {
  margin-right:22px;      /* TODO hack 1/2 */
}
treeview.toggle-like-pixbuf check {
  min-width:22px;         /* TODO hack 2/2 */
  background:transparent;
  border:none;
  -gtk-icon-source: none; /* TODO report bug about HighContrast, that shouldn’t set this */
}
treeview.toggle-like-pixbuf check:checked {
  -gtk-icon-source: -gtk-icontheme("gtk-yes"); /* TODO maybe deprecated; replace with CSS? */
}

/* https://bugzilla.gnome.org/show_bug.cgi?id=744531 */
button.flat:disabled {
  color: @theme_unfocused_bg_color;
}
